==> habu-0.2.0.zip: habu/habu/subscription/config.py <==
# subscription by config
import sets
from twisted.python import log
from twisted.internet import reactor
from twisted.internet.defer import Deferred
from twisted.web.client import getPage
from habu import habuutils
import feedparser


class FeedConfig(object):
    def __init__(self, feedConfig):
        self.feedList = feedConfig
    
    def fetch(self):
        return self.feedList

class FileConfig(object):
    def __init__(self, filePath):
        self.path = filePath

    def fetch(self):
        lines = open(self.path).readlines()
        return [ l.strip() for l in lines]

class OpmlConfig(object):
    def __init__(self, conf):
        pass

    def fetch(self):
        return []

class Config(object):
    def __init__(self, config, environ):
        self.subModules = []
        self.fetchConfigs = []
        feedConfig = config.setdefault("feed", [])[:]
        if feedConfig:
            self.fetchConfigs.append(FeedConfig(feedConfig))
        feedConfig = config.setdefault("file", None)
        if feedConfig:
            self.fetchConfigss.append(FileConfig(feedConfig))
        self.maxThread = config.setdefault("max-thread", 0)

    def setSubModules(self, subModules):
        self.subModules = subModules
        
    def execute(self, content=None):
        feeds = []
        for config in self.fetchConfigs:
            feeds.extend(config.fetch())
        self.feed = sets.Set(feeds)
        try:
            self.feed.remove('')
        except:
            pass

        self.deferredList = [Deferred()  for f in self.feed]

        maxThread = min(
            self.maxThread > 0 and self.maxThread or len(self.feed),
            len(self.feed))

        for i in range(0, maxThread):
            url = self.feed.pop()
            getPage(url).addCallback(
                self.gotPage).addErrback(
                self.gotErr)

        result = self.deferredList[:]
        result = habuutils.mergeResult(result, self.subModules)

        return result

    def gotErr(self, failure):
        deferred = self.deferredList.pop()
        deferred.errback(failure)

    def gotPage(self, content):
        feed = feedparser.parse(content)
        if self.feed != None:
            deferred = self.deferredList.pop()
            deferred.callback(feed)
            if self.feed:
                url = self.feed.pop()
                log.msg("subscription.config : Next Feed Fetcher Start")
                getPage(url).addCallback(
                    self.gotPage).addErrback(
                    self.gotErr)
        else:
            deferred = self.deferredList.pop()
            deferred.errback("tmiki.subscription.config: parse failed")

        return content


def create(config, environ):
    return Config(config, environ)

==> habu-0.3.zip: habu/habu/subscription/config.py <==
# subscription by config
import sets
from twisted.internet import reactor
from twisted.internet.defer import Deferred
from habu import habuutils
from habu.webutils import getPage
import feedparser
import habu.log as log


class FeedConfig(object):
    def __init__(self, feedConfig):
        self.feedList = feedConfig
    
    def fetch(self):
        return self.feedList

class FileConfig(object):
    def __init__(self, filePath):
        self.path = filePath

    def fetch(self):
        lines = open(self.path).readlines()
        return [ l.strip() for l in lines]

class OpmlConfig(object):
    def __init__(self, conf):
        pass

    def fetch(self):
        return []

class Config(object):
    def __init__(self, config, environ):
        self.subModules = []
        self.fetchConfigs = []
        feedConfig = config.setdefault("feed", [])[:]
        if feedConfig:
            self.fetchConfigs.append(FeedConfig(feedConfig))
        feedConfig = config.setdefault("file", None)
        if feedConfig:
            self.fetchConfigss.append(FileConfig(feedConfig))
        self.maxThread = config.setdefault("max-thread", 0)
        self.proxy_host = environ.get("proxy_host", None)
        self.proxy_port = environ.get("proxy_port", 0)

    def setSubModules(self, subModules):
        self.subModules = subModules
        
    def execute(self, content=None):
        feeds = []
        for config in self.fetchConfigs:
            feeds.extend(config.fetch())
        self.feed = sets.Set(feeds)
        try:
            self.feed.remove('')
        except:
            pass

        self.deferredList = [Deferred()  for f in self.feed]

        maxThread = min(
            self.maxThread > 0 and self.maxThread or len(self.feed),
            len(self.feed))

        for i in range(0, maxThread):
            url = self.feed.pop()
            getPage(url, self.proxy_host, self.proxy_port).addCallback(
                self.gotPage, url).addErrback(
                self.gotErr)

        result = self.deferredList[:]
        result = habuutils.mergeResult(result, self.subModules)

        return result

    def gotErr(self, failure):
        log.debug("filter.config gotErr")
        deferred = self.deferredList.pop()
        deferred.errback(failure)

    def postParse(self, content):
        feed = content["feed"]
        if not feed.has_key("link") and feed.get("id", None):
            feed["link"] = feed.get("id")
        
    def gotPage(self, content, url):
        log.debug("filter.config gotPage")
        feed = feedparser.parse(content)
        if feed  and feed.get("entries"):
            self.postParse(feed)
            deferred = self.deferredList.pop()
            deferred.callback(feed)
        else:
            deferred = self.deferredList.pop()
            #log.debug(content)
            open("cnet.rss", "wb").write(content)
            error_message = "tmiki.subscription.config: parse failed for " + url
            if feed and feed.get("bozo_exception"):
                error_message += " " + str(feed.get("bozo_exception"))
            deferred.errback(error_message)

        if self.feed:
            nextUrl = self.feed.pop()
            log.debug("subscription.config : Next Feed Fetcher Start")
            getPage(nextUrl, self.proxy_host, self.proxy_port).addCallback(
                self.gotPage, nextUrl).addErrback(
                self.gotErr)

        return content


def create(config, environ):
    return Config(config, environ)
